@@ -1,10 +1,10 @@
 package Zonemaster::Engine::Exception;
 
-use version; our $VERSION = version->declare("v1.0.3");
-
-use 5.014002;
+use v5.16.0;
 use warnings;
 
+use version; our $VERSION = version->declare("v1.0.3");
+
 use Class::Accessor "antlers";
 
 use overload '""' => \&string;
